@@ -4,7 +4,7 @@
 de.util
 ~~~~~~~~~~~
 
-:2019 by Robin Schwemmle.
+:2021 by Robin Schwemmle.
 :license: GNU GPLv3, see LICENSE for more details.
 """
 
@@ -22,8 +22,8 @@ from de import de
 sns.set_style("ticks", {"xtick.major.size": 8, "ytick.major.size": 8})
 
 
-_mmd = r"[mm $d^{-1}$]"
-_m3s = r"[$m^{3}$ $s^{-1}$]"
+_mmd = r"Q [mm $d^{-1}$]"
+_m3s = r"Q [$m^{3}$ $s^{-1}$]"
 _q_lab = _mmd
 _sim_lab = "Simulated"
 
@@ -53,7 +53,6 @@ def import_ts(path, sep=","):
 
     return df_ts
 
-
 def import_camels_ts(path, sep=r"\s+", catch_area=None):
     r"""
     Import .txt-file with streamflow time series from CAMELS dataset (cubic
@@ -217,8 +216,10 @@ def fdc_obs_sim_ax(obs, sim, ax, fig_num):  # pragma: no cover
     ----------
     obs : series
         observed time series
+
     sim : series
         simulated time series
+
     ax : axes
         Axes object to draw the plot onto
     fig_num : string
@@ -253,10 +254,13 @@ def plot_obs_sim_ax(obs, sim, ax, fig_num):  # pragma: no cover
     ----------
     obs : series
         observed time series
+
     sim : series
         simulated time series
+
     ax : axes
         Axes object to draw the plot onto
+
     fig_num : string
         string object for figure caption
     """
@@ -278,7 +282,8 @@ def plot_obs_sim_ax(obs, sim, ax, fig_num):  # pragma: no cover
 
 
 def diag_polar_plot_multi_fc(
-    brel_mean, b_area, temp_cor, sig_de, b_dir, diag, fc, l=0.05, ax_lim=-0.6
+    brel_mean, temp_cor, eff_de, b_dir, phi, b_hf, b_lf, b_tot,
+    err_hf, err_lf, fc, l=0.05, ax_lim=1.2
 ):  # pragma: no cover
     r"""Multiple polar plot of Diagnostic-Efficiency (DE)
 
@@ -287,21 +292,33 @@ def diag_polar_plot_multi_fc(
     brel_mean : (N,)array_like
         relative mean bias as 1-D array
 
-    b_area : (N,)array_like
-        bias area as 1-D array
-
     temp_cor : (N,)array_like
         temporal correlation as 1-D array
 
-    sig_de : (N,)array_like
+    eff_de : (N,)array_like
         diagnostic efficiency as 1-D array
 
     b_dir : (N,)array_like
         direction of bias as 1-D array
 
-    diag : (N,)array_like
+    phi : (N,)array_like
         angle as 1-D array
 
+    b_hf : (N,)array_like
+        high flow bias
+
+    b_lf : (N,)array_like
+        low flow bias
+
+    b_tot : (N,)array_like
+        absolute total relative bias
+
+    err_hf : (N,)array_like
+        contribution of high flow errors
+
+    err_lf : (N,)array_like
+        contribution of low flow errors
+
     fc : list
         figure captions
 
@@ -319,14 +336,18 @@ def diag_polar_plot_multi_fc(
 
         \varphi = arctan2(\overline{B_{rel}}, B_{slope})
     """
-    sig_min = np.min(sig_de)
+    de_max = np.min(eff_de)
 
     ll_brel_mean = brel_mean.tolist()
-    ll_b_dir = b_dir.tolist()
-    ll_b_area = b_area.tolist()
-    ll_sig = sig_de.tolist()
-    ll_diag = diag.tolist()
     ll_temp_cor = temp_cor.tolist()
+    ll_eff = eff_de.tolist()
+    ll_b_dir = b_dir.tolist()
+    ll_phi = phi.tolist()
+    ll_b_hf = b_hf.tolist()
+    ll_b_lf = b_lf.tolist()
+    ll_b_tot = b_tot.tolist()
+    ll_err_hf = err_hf.tolist()
+    ll_err_lf = err_lf.tolist()
 
     # convert temporal correlation to color
     norm = matplotlib.colors.Normalize(vmin=0, vmax=1.0)
@@ -334,8 +355,8 @@ def diag_polar_plot_multi_fc(
     delta = 0.01  # for spacing
 
     # determine axis limits
-    yy = np.arange(ax_lim, 1.01, delta)[::-1]
-    c_levels = np.arange(ax_lim, 1, 0.2)
+    yy = np.arange(0.01, ax_lim, delta)
+    c_levels = np.arange(0, ax_lim, 0.2)
 
     len_yy = 360
     # len_yy1 = 90
@@ -355,32 +376,32 @@ def diag_polar_plot_multi_fc(
     ax.cla()
     # plot regions
     ax.plot(
-        (1, np.deg2rad(45)),
-        (1, np.min(yy)),
+        (0, np.deg2rad(45)),
+        (0, np.max(yy)),
         color="lightgray",
         linewidth=1.5,
         ls="--",
         zorder=0,
     )
     ax.plot(
-        (1, np.deg2rad(135)),
-        (1, np.min(yy)),
+        (0, np.deg2rad(135)),
+        (0, np.max(yy)),
         color="lightgray",
         linewidth=1.5,
         ls="--",
         zorder=0,
     )
     ax.plot(
-        (1, np.deg2rad(225)),
-        (1, np.min(yy)),
+        (0, np.deg2rad(225)),
+        (0, np.max(yy)),
         color="lightgray",
         linewidth=1.5,
         ls="--",
         zorder=0,
     )
     ax.plot(
-        (1, np.deg2rad(315)),
-        (1, np.min(yy)),
+        (0, np.deg2rad(315)),
+        (0, np.max(yy)),
         color="lightgray",
         linewidth=1.5,
         ls="--",
@@ -390,28 +411,45 @@ def diag_polar_plot_multi_fc(
     cp = ax.contour(theta, r, r, colors="darkgray", levels=c_levels, zorder=1)
     cl = ax.clabel(cp, inline=False, fontsize=10, fmt="%1.1f", colors="dimgrey")
     # threshold efficiency for FBM
-    sig_l = 1 - np.sqrt((l) ** 2 + (l) ** 2 + (l) ** 2)
+    eff_l = np.sqrt((l) ** 2 + (l) ** 2 + (l) ** 2)
     # loop over each data point
-    for (bm, bd, ba, r, sig, ang, txt) in zip(
-        ll_brel_mean, ll_b_dir, ll_b_area, ll_temp_cor, ll_sig, ll_diag, fc
-    ):
-        # slope of bias
-        b_slope = de.calc_bias_slope(ba, bd)
+    zz = zip(ll_brel_mean, ll_temp_cor, ll_eff, ll_b_dir, ll_phi, ll_b_hf,
+             ll_b_lf, ll_b_tot, ll_err_hf, ll_err_lf, fc)
+    for (bm, r, eff, bd, ang, bhf, blf, btot, errhf, errlf, txt) in zz:
         # convert temporal correlation to color
         rgba_color = cm.plasma_r(norm(r))
-        # relation of b_dir which explains the error
-        if abs(ba) > 0:
-            exp_err = (abs(bd) * 2) / abs(ba)
-        elif abs(ba) == 0:
+        # relation of high flow errors and low flow errors which explain
+        # the total FDC error
+        if btot > 0:
+            exp_err = (abs(bhf) + abs(blf)) / btot
+        elif btot == 0:
             exp_err = 0
 
+        # calculate pies to display error contribution of high flows and low
+        # flows
+        # calculate the points of the first pie marker
+        # these are just the origin (0, 0) + some (cos, sin) points on a circle
+        r1 = abs(errhf)/2
+        x1 = np.cos(2 * np.pi * np.linspace(0.25 - r1/2, 0.25 + r1/2))
+        y1 = np.sin(2 * np.pi * np.linspace(0.25 - r1/2, 0.25 + r1/2))
+        xy1 = np.row_stack([[0, 0], np.column_stack([x1, y1])])
+        s1 = np.abs(xy1).max()
+
+        r2 = abs(errlf)/2
+        x2 = np.cos(2 * np.pi * np.linspace(0.75 - r2/2, 0.75 + r2/2))
+        y2 = np.sin(2 * np.pi * np.linspace(0.75 - r2/2, 0.75 + r2/2))
+        xy2 = np.row_stack([[0, 0], np.column_stack([x2, y2])])
+        s2 = np.abs(xy2).max()
+
         # diagnose the error
-        if abs(bm) <= l and exp_err > l and sig <= sig_l:
-            c = ax.scatter(ang, sig, s=75, color=rgba_color, zorder=3)
-            d = ax.scatter(ang, sig, color="grey", marker=".", zorder=4)
+        if abs(bm) <= l and exp_err > l and eff > eff_l:
+            c0 = ax.scatter(ang, eff, marker=xy1, s=s1**2 * 200, facecolor=rgba_color, zorder=3)
+            c1 = ax.scatter(ang, eff, marker=xy2, s=s2**2 * 200, facecolor=rgba_color, zorder=3)
+            c = ax.scatter(ang, eff, s=36, facecolor=rgba_color, zorder=3)
+            c2 = ax.scatter(ang, eff, s=4, facecolor='grey', zorder=3)
             ax.annotate(
                 txt,
-                xy=(ang, sig),
+                xy=(ang, eff),
                 color="black",
                 fontsize=13,
                 xytext=(-8, 0),
@@ -419,12 +457,14 @@ def diag_polar_plot_multi_fc(
                 ha="center",
                 va="center",
             )
-        elif abs(bm) > l and exp_err <= l and sig <= sig_l:
-            c = ax.scatter(ang, sig, s=75, color=rgba_color, zorder=3)
-            d = ax.scatter(ang, sig, color="grey", marker=".", zorder=4)
+        elif abs(bm) > l and exp_err <= l and eff > eff_l:
+            c0 = ax.scatter(ang, eff, marker=xy1, s=s1**2 * 200, facecolor=rgba_color, zorder=3)
+            c1 = ax.scatter(ang, eff, marker=xy2, s=s2**2 * 200, facecolor=rgba_color, zorder=3)
+            c = ax.scatter(ang, eff, s=36, facecolor=rgba_color, zorder=3)
+            c2 = ax.scatter(ang, eff, s=4, facecolor='grey', zorder=3)
             ax.annotate(
                 txt,
-                xy=(ang, sig),
+                xy=(ang, eff),
                 color="black",
                 fontsize=13,
                 xytext=(-8, 0),
@@ -432,12 +472,14 @@ def diag_polar_plot_multi_fc(
                 ha="center",
                 va="center",
             )
-        elif abs(bm) > l and exp_err > l and sig <= sig_l:
-            c = ax.scatter(ang, sig, s=75, color=rgba_color, zorder=3)
-            d = ax.scatter(ang, sig, color="grey", marker=".", zorder=4)
+        elif abs(bm) > l and exp_err > l and eff > eff_l:
+            c0 = ax.scatter(ang, eff, marker=xy1, s=s1**2 * 200, facecolor=rgba_color, zorder=3)
+            c1 = ax.scatter(ang, eff, marker=xy2, s=s2**2 * 200, facecolor=rgba_color, zorder=3)
+            c = ax.scatter(ang, eff, s=36, facecolor=rgba_color, zorder=3)
+            c2 = ax.scatter(ang, eff, s=4, facecolor='grey', zorder=3)
             ax.annotate(
                 txt,
-                xy=(ang, sig),
+                xy=(ang, eff),
                 color="black",
                 fontsize=13,
                 xytext=(-8, 0),
@@ -445,25 +487,26 @@ def diag_polar_plot_multi_fc(
                 ha="center",
                 va="center",
             )
+
         # FBM
-        elif abs(bm) <= l and exp_err <= l and sig <= sig_l:
+        elif abs(bm) <= l and exp_err <= l and eff > eff_l:
             ax.annotate(
                 "",
-                xytext=(0, 1),
-                xy=(0, sig),
+                xytext=(0, 0),
+                xy=(0, eff),
                 arrowprops=dict(edgecolor=rgba_color, facecolor="black", lw=3),
                 zorder=2,
             )
             ax.annotate(
                 "",
-                xytext=(0, 1),
-                xy=(np.pi, sig),
+                xytext=(0, 0),
+                xy=(np.pi, eff),
                 arrowprops=dict(edgecolor=rgba_color, facecolor="black", lw=3),
                 zorder=2,
             )
             ax.annotate(
                 txt,
-                xy=(ang, sig),
+                xy=(ang, eff),
                 color="black",
                 fontsize=13,
                 xytext=(-8, 0),
@@ -472,12 +515,12 @@ def diag_polar_plot_multi_fc(
                 va="center",
             )
         # FGM
-        elif abs(bm) <= l and exp_err <= l and sig > sig_l:
-            c = ax.scatter(ang, sig, s=75, color=rgba_color, zorder=3)
-            d = ax.scatter(ang, sig, color="grey", marker=".", zorder=4)
+        elif abs(bm) <= l and exp_err <= l and eff <= eff_l:
+            c = ax.scatter(ang, eff, color=rgba_color, zorder=3)
+            d = ax.scatter(ang, eff, color="grey", marker=".", zorder=4)
             ax.annotate(
                 txt,
-                xy=(ang, sig),
+                xy=(ang, eff),
                 color="black",
                 fontsize=13,
                 xytext=(-6, 0),
@@ -485,8 +528,27 @@ def diag_polar_plot_multi_fc(
                 ha="center",
                 va="center",
             )
+
+    # legend for error contribution of high flows and low flows
+    rl1 = 1/2
+    xl1 = np.cos(2 * np.pi * np.linspace(0.25 - rl1/2, 0.25 + rl1/2))
+    yl1 = np.sin(2 * np.pi * np.linspace(0.25 - rl1/2, 0.25 + rl1/2))
+    xyl1 = np.row_stack([[0, 0], np.column_stack([xl1, yl1])])
+    sl1 = np.abs(xyl1).max()
+
+    rl2 = 1/2
+    xl2 = np.cos(2 * np.pi * np.linspace(0.75 - rl2/2, 0.75 + rl2/2))
+    yl2 = np.sin(2 * np.pi * np.linspace(0.75 - rl2/2, 0.75 + rl2/2))
+    xyl2 = np.row_stack([[0, 0], np.column_stack([xl2, yl2])])
+    sl2 = np.abs(xyl2).max()
+    ax.scatter([], [], color='k', zorder=2, marker=xyl1, s=sl1**2 * 200, label=r'high flows ($\epsilon_{hf}=1$)')
+    ax.scatter([], [], color='k', zorder=2, marker=xyl2, s=sl2**2 * 200, label=r'low flows ($\epsilon_{lf}=1$)')
+    ax.legend(loc='upper right', title="Error contribution of", fancybox=False,
+              frameon=False, bbox_to_anchor=(1.3, 1.1), title_fontsize=11,
+              fontsize=11, handletextpad=0.1)
+
     ax.set_rticks([])  # turn default ticks off
-    ax.set_rmin(1)
+    ax.set_rmin(0)
     ax.set_rmax(ax_lim)
     ax.tick_params(
         labelleft=False,
@@ -495,7 +557,7 @@ def diag_polar_plot_multi_fc(
         labelbottom=True,
         grid_alpha=0.01,
     )  # turn labels and grid off
-    ax.set_xticklabels(["", "", "", "", "", "", "", ""])
+    ax.set(xticklabels=[])
     ax.text(
         -0.14,
         0.5,
@@ -600,7 +662,7 @@ def diag_polar_plot_multi_fc(
     cbar = fig.colorbar(
         dummie_cax, ax=ax, orientation="horizontal", ticks=[1, 0.5, 0], shrink=0.8
     )
-    cbar.set_label(r"r [-]", fontsize=12, labelpad=8)
+    cbar.set_label("r [-]\nTiming error", labelpad=4)
     cbar.set_ticklabels(["1", "0.5", "<0"])
     cbar.ax.tick_params(direction="in", labelsize=10)
 
@@ -635,12 +697,12 @@ def polar_plot_multi_fc(
     fig : Figure
         diagnostic polar plot
     """
-    sig_min = np.min(sig_kge)
+    de_max = np.min(sig_kge)
 
     ll_kge_beta = kge_beta.tolist()
     ll_ag = alpha_or_gamma.tolist()
     ll_kge_r = kge_r.tolist()
-    ll_sig = sig_kge.tolist()
+    ll_eff = sig_kge.tolist()
 
     # convert temporal correlation to color
     norm = matplotlib.colors.Normalize(vmin=0, vmax=1.0)
@@ -704,15 +766,15 @@ def polar_plot_multi_fc(
     cp = ax.contour(theta, r, r, colors="darkgray", levels=c_levels, zorder=1)
     cl = ax.clabel(cp, inline=False, fontsize=10, fmt="%1.1f", colors="dimgrey")
     # loop over each data point
-    for (b, ag, r, sig, txt) in zip(ll_kge_beta, ll_ag, ll_kge_r, ll_sig, fc):
+    for (b, ag, r, eff, txt) in zip(ll_kge_beta, ll_ag, ll_kge_r, ll_eff, fc):
         ang = np.arctan2(b - 1, ag - 1)
         # convert temporal correlation to color
         rgba_color = cm.plasma_r(norm(r))
-        c = ax.scatter(ang, sig, s=75, color=rgba_color, zorder=2)
-        d = ax.scatter(ang, sig, color="grey", marker=".", zorder=4)
+        c = ax.scatter(ang, eff, s=75, color=rgba_color, zorder=2)
+        d = ax.scatter(ang, eff, color="grey", marker=".", zorder=4)
         ax.annotate(
             txt,
-            xy=(ang, sig),
+            xy=(ang, eff),
             color="black",
             fontsize=13,
             xytext=(8, 0),
@@ -807,7 +869,7 @@ def polar_plot_multi_fc(
         rotation_mode="anchor",
         transform=ax.transAxes,
     )
-    ax.set_xticklabels(["", "", "", "", "", "", "", ""])
+    ax.set(xticklabels=[])
     ax.set_rticks([])  # turn default ticks off
     ax.set_rmin(1)
     ax.set_rmax(ax_lim)
@@ -815,7 +877,7 @@ def polar_plot_multi_fc(
     cbar = fig.colorbar(
         dummie_cax, ax=ax, orientation="horizontal", ticks=[1, 0.5, 0], shrink=0.8
     )
-    cbar.set_label(r"r [-]", fontsize=12, labelpad=8)
+    cbar.set_label("r [-]\nTiming error", labelpad=4)
     cbar.set_ticklabels(["1", "0.5", "<0"])
     cbar.ax.tick_params(direction="in", labelsize=10)
 
